package org.rcisoft.business.memLeaveMessage.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.memInfo.bean.MemberInfoRedisBean;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.business.memLeaveMessage.bean.MemLeaveMessageRedisBean;
import org.rcisoft.business.memLeaveMessage.dao.MemLeaveMessageRepository;
import org.rcisoft.business.memLeaveMessage.entity.*;
import org.rcisoft.business.memLeaveMessage.service.MemLeaveMessageService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.service.impl.CyRedisServiceImpl;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.impl.DictionaryServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * Created by cy on 2024年12月20日 下午3:35:34.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemLeaveMessageServiceImpl extends ServiceImpl<MemLeaveMessageRepository, MemLeaveMessage>  implements MemLeaveMessageService {



    @Autowired
    CyIdGenUtil cyIdGenUtil;

    @Autowired
    private MemLeaveMessageRepository memLeaveMessageRepository;

    @Autowired
    private DictionaryServiceImpl dictionaryService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private CyRedisServiceImpl cyRedisServiceImpl;

    @Autowired
    private MemInfoRepository memInfoRepository;
    /**
     * 留言
     * @param memLeaveMessage
     * @return
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public CyPersistModel leaveMessage(MemLeaveMessage memLeaveMessage) {
        String userId = CyUserUtil.getAuthenBusinessId();
        //获取留言每天限制条数
        List<DictData> leaveMessageConfig = dictionaryService.selectByTypes("leave_message_config");
        DictData dictData1 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("leave_message_limit_count")).findFirst().orElse(null);
        DictData dictData2 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("pay_leave_message_limit_count")).findFirst().orElse(null);
        Integer leaveMessageLimitCount = Integer.valueOf(dictData1.getDictValue());
        Integer payLeaveMessageLimitCount = Integer.valueOf(dictData2.getDictValue());
        //判断redis中的留言次数是否大于等于leaveMessageLimitCount
        Object redisLeaveMessageCount = cyRedisServiceImpl.hget(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + userId, String.valueOf(memLeaveMessage.getTargetId()));
        //超出留言次数的条件：
        //1:redis中的存储数量>=字典限制数量
        //2:用户不用金币进行留言
        //3:超出金币进行留言的限制数量
        if (redisLeaveMessageCount != null && Integer.parseInt(redisLeaveMessageCount.toString()) >= leaveMessageLimitCount && memLeaveMessage.getIsPay().equals("0")){
            //超出留言次数
            if (Integer.parseInt(redisLeaveMessageCount.toString()) >= payLeaveMessageLimitCount){
                //超出金币进行留言的数量限制
                throw new CyServiceException(1002,"今日留言次数超出限制");
            } else {
                throw new CyServiceException(1001,"留言次数超出限制");
            }
        } else {
            if (Integer.parseInt(redisLeaveMessageCount.toString()) >= payLeaveMessageLimitCount){
                //超出金币进行留言的数量限制
                throw new CyServiceException(1002,"今日留言次数超出限制");
            }
            //先插入到留言表
            memLeaveMessage.setCreateBy(Integer.valueOf(userId));
            memLeaveMessage.setUpdateBy(Integer.valueOf(userId));
            memLeaveMessage.setCreateDate(new Date());
            memLeaveMessage.setUpdateDate(new Date());
            int line = memLeaveMessageRepository.insert(memLeaveMessage);
            //修改对话表
            //查询是否已存在对话
            MemLeaveMessage memUserTalks = memLeaveMessageRepository.queryUserTalk(Integer.valueOf(userId),memLeaveMessage.getTargetId());
            if (memUserTalks != null){
                //如果已经存在该对话，则只更新信息
                MemLeaveMessage memUserTalk = new MemLeaveMessage();
                memUserTalk.setBusinessId(memUserTalks.getBusinessId());
                memUserTalk.setCreateBy(Integer.valueOf(userId));
                memUserTalk.setUpdateBy(Integer.valueOf(userId));
                memUserTalk.setTargetId(memLeaveMessage.getTargetId());
                memUserTalk.setContent(memLeaveMessage.getContent());
                memUserTalk.setCreateDate(new Date());
                memUserTalk.setUpdateDate(new Date());
                memUserTalk.setDelStatus("0");
                int line1 = memLeaveMessageRepository.updateUserTalk(memUserTalk);
                if (line1 > 0) {
                    //插入对话表成功，则更新redis
                    this.redisTemplate.opsForHash().increment(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + userId, String.valueOf(memLeaveMessage.getTargetId()), (double) 1L);
                    //底下的暂时有bug 暂时无法解决
                    //cyRedisServiceImpl.hincr(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + userId, String.valueOf(memLeaveMessage.getTargetId()), (double) 1L);
                }
            } else {
                MemLeaveMessage memUserTalk = new MemLeaveMessage();
                memUserTalk.setCreateBy(Integer.valueOf(userId));
                memUserTalk.setUpdateBy(Integer.valueOf(userId));
                memUserTalk.setCreateDate(new Date());
                memUserTalk.setUpdateDate(new Date());
                memUserTalk.setTargetId(memLeaveMessage.getTargetId());
                memUserTalk.setContent(memLeaveMessage.getContent());
                memUserTalk.setDelStatus("0");
                int line1 = memLeaveMessageRepository.insertUserTalk(memUserTalk);
                if (line1 > 0){
                    //插入对话表成功，则更新redis
                    cyRedisServiceImpl.hset(MemLeaveMessageRedisBean.USER_LEAVE_MESSAGE + userId, String.valueOf(memLeaveMessage.getTargetId()), 1L);
                }
            }
            //金币支付进行留言
            if (memLeaveMessage.getIsPay().equals("1")){
                //扣除该用户的金币数量
                DictData dictData3 = leaveMessageConfig.stream().filter(item -> item.getDictLabel().equals("pay_count")).findFirst().orElse(null);
                Integer payCount = Integer.valueOf(dictData3.getDictValue());
                MemInfo memInfo = memInfoRepository.selectByOpenId(memLeaveMessage.getWxOpenid());
                memInfo.setGoldCoinsCount(memInfo.getGoldCoinsCount() - payCount);
                memInfoRepository.updateById(memInfo);
            }
            return new CyPersistModel(line);
        }
    }

    /**
     * 分页查询对话列表
     *
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> queryTalkByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        memLeaveMessage.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.queryTalkByPagination(paginationUtility, memLeaveMessage);
        return userLeaveMessageIPage;
    }

    /**
     * 查询留言列表详情 1对1的
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> leaveMessageDetailByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        memLeaveMessage.setCreateBy(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        //先按倒序取最新的十条
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.leaveMessageDetailList(paginationUtility, memLeaveMessage);
        //再对这最新的10条正序排序
        userLeaveMessageIPage.getRecords().sort(Comparator.comparing(MemLeaveMessage::getCreateDate));
        return userLeaveMessageIPage;
    }

    /**
     * 分页查询留言列表
     *
     * @param paginationUtility
     * @param memLeaveMessage
     */
    @Override
    public IPage<MemLeaveMessage> queryMessageByPagination(CyPageInfo<MemLeaveMessage> paginationUtility, MemLeaveMessage memLeaveMessage) {
        IPage<MemLeaveMessage> userLeaveMessageIPage = memLeaveMessageRepository.queryMessageByPagination(paginationUtility, memLeaveMessage);
        return userLeaveMessageIPage;
    }
}
